#include "StdAfx.h"
#include "RMOD_K_Formula_Opti.h"

CRMOD_K_Formula_Opti::CRMOD_K_Formula_Opti()
{
   MemoryStatus    = 0;
	Isallocated     = 0;
	IsLoaded        = 0;
	version         = 201;
	strcpy (DataFileName,  "DesignStart.dat");	
	strcpy (DataResultName,"DesignResults.dat");	
	strcpy (DataCornerName,"CornerStiff.dat");	
	strcpy (OptiFileName,  "DesignTarget.dat");
	strcpy (WorkDirectory, ".\\");
	strcpy (JobTitle,      "   ");
   myDiscreteFormula = new CDiscreteFormula;
   myContinusFormula = new CContinusFormula;
}
CRMOD_K_Formula_Opti::~CRMOD_K_Formula_Opti()
{
	delete myDiscreteFormula;
   delete myContinusFormula;
}
double CRMOD_K_Formula_Opti::Set_TargetValue ( double *design, double *modeofcall )
//-------------------------------------------------------------------------------
// --- run optimization
//-------------------------------------------------------------------------------
{
   FILE   *ResultFile;
	FILE   *GNUFILE;
   char   s[1024],sfile[1024];
   int    i,j,k,targetpoints,iFx,iFy,ix,iy;
   double target,targetfactor,wheelload;
   double foutput[Noutput];
//
// --- build data base for target value
//
   if ( ( int )( *modeofcall ) == 0 )
   {
      MemoryStatus = 1;
//
// --- read target data
//
     if ( Get_TargetData () == 0 )
      {
         printf ( " error in reading file DesignTarget \n" );
         return ( 1.0e+11 );
      }
#ifndef GUI
//
// --- read design variables and model parameter
//
      NumberOfDesignVariables = 0;
      if ( ReadFile_Formula ( design ) == 0 )
      {
         printf ( " error in reading file DesignStart \n" );
         return ( 1.0e+12 );
      }
#endif
//
// --- init done
//
      if ( ( int )( *modeofcall ) == 0 ) return ( NumberOfDesignVariables );
   }
//
// --- check if init done
//
   if ( MemoryStatus == 0 )
   {
      printf ( " call function (design,0) first \n" );
      return ( 1.0e+12 );
   }
//
// --- call model for setting design variables
//
   else
   {
//
// --- call model
//
#ifndef GUI
      if      ( model == 1 ) myContinusFormula->ContinusFormula_SetVariables ( input1, design, isdesvar );
      else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_SetVariables ( input1, design, isdesvar );
#else
      if      ( model == 1 ) myContinusFormula->ContinusFormula_SetVariables ( input1, Nmodelparameter );
      else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_SetVariables ( input1, Nmodelparameter );
#endif
   }
//
// --- call model for target computation
//
   targetvalue  = 0.0;
	targetpoints = 0;
   iFx          = 0;
   iFy          = 0;
   ix           = 0;
   iy           = 0;

   j  = 0;
   for ( i=0; i<NumberOfFiles; i++ )
   {
      if  ( TargetDirectionPerFile[i] == 1 ) iFx++;
      if  ( TargetDirectionPerFile[i] == 2 ) iFy++;

      for ( k=0; k<PointsPerFile[i]; k++ )
      {
         input3[0] = optidata [j  ];
         input3[1] = optidata [j+1];
         input3[2] = optidata [j+2];
         input3[3] = optidata [j+5];
	      input3[4] = optidata [j+7];
//
// --- check for zero or negative wheel loads
//
			targetfactor = 1000.0;
			if ( optidata [j+5] < 2000.0e+00 ) 
			{
				targetfactor   = 0.0;
				wheelload      = 1.0e-05;
			}
			else 
			{
				wheelload = optidata [j+5];
				targetpoints++;
			}

         if      ( model == 1 ) myContinusFormula->ContinusFormula_Evaluate ( input3,output );
         else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_Evaluate ( input3,output );
//
// --- force and
//
         if      ( TargetDirectionPerFile[i] == 1 )
         {
            target = sqrt( ( output[0]-optidata [j+3] )*( output[0]-optidata [j+3] ) ) / wheelload;
				resultdata[j  ] = output[0];
				resultdata[j+3] = output[3];
				resultdata[j+4] = output[4];
         }
         else if ( TargetDirectionPerFile[i] == 2 )
         {
            target = sqrt( ( output[1]-optidata [j+4] )*( output[1]-optidata [j+4] ) ) / wheelload;
				resultdata[j+1] = output[1];
				resultdata[j+5] = output[5];
				resultdata[j+6] = output[6];
			}
         else if ( TargetDirectionPerFile[i] > 2 )
         {
            target = sqrt( ( output[0]-optidata [j+3] )*( output[0]-optidata [j+3] ) ) / wheelload +
                     sqrt( ( output[1]-optidata [j+4] )*( output[1]-optidata [j+4] ) ) / wheelload;

				target = sqrt( ( output[0]-optidata [j+3] )*( output[0]-optidata [j+3] )  +
                           ( output[1]-optidata [j+4] )*( output[1]-optidata [j+4] ) ) / wheelload;

				target = sqrt( ( output[0]-optidata [j+3] )*( output[0]-optidata [j+3] )  +
                           ( output[1]-optidata [j+4] )*( output[1]-optidata [j+4] ) );

				resultdata[j  ] = output[0];
			   resultdata[j+1] = output[1];
			   resultdata[j+2] = output[2];
				resultdata[j+3] = output[3];
				resultdata[j+4] = output[4];
				resultdata[j+5] = output[5];
				resultdata[j+6] = output[6];
//
// --- moment
//
            if ( TargetDirectionPerFile[i] > 3 )
            {
               target = target + sqrt( ( output[2]-optidata [j+6] )*( output[2]-optidata [j+6] ) ) / ( 500.0*wheelload );
					resultdata[j+2] = output[2];
            }
         }
			if ( model == 2 ) DA[i] = myDiscreteFormula->get_dA();
//
// --- build target
//
         targetvalue = targetvalue + targetfactor*target;
			resultdata[j+8] = targetfactor*target;
			optidata  [j+8] = targetvalue;

         //for ( int ii=0; ii<7; ii++ ) 
         //{
         //   if ( fabs(output[ii])       > 20000000.0 ) System::Windows::Forms::MessageBox::Show(" output     = " + output[ii].ToString()       + " at ii = " + ii.ToString() + " at i = " + i.ToString() + " at j = " + j.ToString(),"Hint");
         //   if ( fabs(resultdata[j+ii]) > 20000000.0 ) System::Windows::Forms::MessageBox::Show(" resultdata = " + resultdata[j+ii].ToString() + " at ii = " + ii.ToString() + " at i = " + i.ToString() + " at j = " + j.ToString(),"Hint");
         //}

         j  = j  + (Ninput+1);
      }
   }
   if ( targetpoints > 0 ) targetvalue = 0.02*targetvalue/targetpoints;
	else                    targetvalue = 0.02*targetvalue;
// 
// --- write results in files for plotting
//
   if ( ( int )( *modeofcall ) == -1 )
   {
#ifndef GUI
      if      ( model == 1 ) myContinusFormula->ContinusFormula_SetVariables ( input1, design, isdesvar );
      else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_SetVariables ( input1, design, isdesvar );
#else
      if      ( model == 1 ) myContinusFormula->ContinusFormula_SetVariables ( input1, Nmodelparameter );
      else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_SetVariables ( input1, Nmodelparameter );
#endif
//
// --- gnufile header
//
		if (   ( GNUFILE = fopen ("OptimisationResults.gp","w+") ) == NULL ) return (10000.0);
		fprintf (GNUFILE,"reset\n");
		fprintf (GNUFILE,"set term win\n");
		fprintf (GNUFILE,"set grid\n");
//
// --- for each file
//
      j = 0;
      for ( i=0; i<NumberOfFiles; i++ )
      {
         sprintf ( sfile,"file%d.dat",i+1 );
         if ( ( ResultFile = fopen ( sfile,"w+" ) ) == NULL )
         {
            printf ( " Can't open result file number %d\n",i );
         }
         for ( k=0; k<PointsPerFile[i]; k++ )
         {
            input3[0] = optidata [j  ];
            input3[1] = optidata [j+1];
            input3[2] = optidata [j+2];
            input3[3] = optidata [j+5];
            input3[4] = optidata [j+7];

            if      ( model == 1 ) myContinusFormula->ContinusFormula_Evaluate( input3,foutput );
            else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_Evaluate( input3,foutput );
//
// --- results:  1-sl 2-sq 3-st 4-Fz 5-Vt 6-Fxmea 7-Fymea 8-Mzmea 9-Fxsim 10-Fysim 11-Mzsim  
//
            fprintf ( ResultFile,"%+12.6e %+12.6e %+12.6e %+12.6e %+12.6e ",input3[0],input3[1],input3[2],input3[3],input3[4] );
            fprintf ( ResultFile,"%+12.6e %+12.6e %+12.6e  ",optidata [j+3],optidata [j+4],optidata [j+6] );
            fprintf ( ResultFile,"%+12.6e %+12.6e %+12.6e\n",foutput    [0],foutput    [1],foutput    [2] );

            j = j + (Ninput+1);
         }
         fclose( ResultFile );
      }
//
// --- gnuplot stuff Fx
//
		fprintf (GNUFILE," \n");	
		fprintf (GNUFILE,"set ylabel \"Fx [N]\"\n");
		fprintf (GNUFILE,"set xlabel \"longitudinal slip\"\n");
      fprintf (GNUFILE,"plot \\\n");
      for ( i=0; i<NumberOfFiles; i++ )
      {
			if  ( TargetDirectionPerFile[i] == 1 )
			{
            ix++;
            sprintf ( s,"file%d.dat",i+1 );
				fprintf (GNUFILE,"\"%s\" u 1:($6) title \"Mea\"    w l  lt 1 lw 3,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 1:($6*1.05)   notitle w l  lt 6 lw 1,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 1:($6*0.95)   notitle w l  lt 6 lw 1,\\\n",s);

				if ( ix < iFx )
				{
					fprintf (GNUFILE,"\"%s\" u 1:($9) title \"Sim\"    w lp lt 2 lw 3,\\\n",s);
            }
				else
				{
					fprintf (GNUFILE,"\"%s\" u 1:($9) title \"Sim\"    w lp lt 2 lw 3\n",s);
		         fprintf (GNUFILE,"pause -1\n");
					fprintf (GNUFILE," \n");
		         fprintf (GNUFILE,"#set term postscript eps enhanced color solid \"Arial\" 14\n");
		         fprintf (GNUFILE,"#set output \"Fx-Opti_Results.eps\" \n");
		         fprintf (GNUFILE,"#rep\n");
		         fprintf (GNUFILE,"#set output\n");
		         fprintf (GNUFILE,"#set term win \n");				
				}
			}
      }
//
// --- gnuplot stuff Fy
//
		fprintf (GNUFILE," \n");	
		fprintf (GNUFILE,"set ylabel \"Fy [N]\"\n");
		fprintf (GNUFILE,"set xlabel \"lateral slip\"\n");
      fprintf (GNUFILE,"plot \\\n");
      for ( i=0; i<NumberOfFiles; i++ )
      {
			if ( TargetDirectionPerFile[i] == 2 )
			{
            iy++;
            sprintf ( s,"file%d.dat",i+1 );
				fprintf (GNUFILE,"\"%s\" u 2:($7)  title \"Mea\"    w l  lt 1 lw 3,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 2:($7*1.05)   notitle  w l  lt 6 lw 1,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 2:($7*0.95)   notitle  w l  lt 6 lw 1,\\\n",s);

				if ( iy < iFy )
				{
					fprintf (GNUFILE,"\"%s\" u 2:($10) title \"Sim\"    w lp lt 2 lw 3,\\\n",s);
				}
				else
				{
					fprintf (GNUFILE,"\"%s\" u 2:($10) title \"Sim\"    w lp lt 2 lw 3\n",s);
		         fprintf (GNUFILE,"pause -1\n");
					fprintf (GNUFILE," \n");
		         fprintf (GNUFILE,"#set term postscript eps enhanced color solid \"Arial\" 14\n");
		         fprintf (GNUFILE,"#set output \"Fy-Opti_Results.eps\" \n");
		         fprintf (GNUFILE,"#rep\n");
		         fprintf (GNUFILE,"#set output\n");
		         fprintf (GNUFILE,"#set term win \n");
				}
			}
      }
      iy = 0;
//
// --- gnuplot stuff Mz
//
		fprintf (GNUFILE," \n");
		fprintf (GNUFILE,"set ylabel \"M [Nmm]\"\n");
		fprintf (GNUFILE,"plot \\\n");
		for ( i=0; i<NumberOfFiles; i++ )
      {
         sprintf ( s,"file%d.dat",i+1 );
			if ( TargetDirectionPerFile[i] == 2 )
			{
            iy++;
				fprintf (GNUFILE,"\"%s\" u 2:($8)  title \"Mea\"    w l  lt 1 lw 3,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 2:($8*1.05)   notitle  w l  lt 6 lw 1,\\\n",s);
				fprintf (GNUFILE,"\"%s\" u 2:($8*0.95)   notitle  w l  lt 6 lw 1,\\\n",s);
              
				if ( iy < iFy )
				{
					fprintf (GNUFILE,"\"%s\" u 2:($11) title \"Sim\"    w lp lt 2 lw 3,\\\n",s);
				}
				else
				{
					fprintf (GNUFILE,"\"%s\" u 2:($11) title \"Sim\"    w lp lt 2 lw 3\n",s);
				}
			}
		}
		fprintf (GNUFILE,"pause -1\n");
		fprintf (GNUFILE," \n");
		fprintf (GNUFILE,"#set term postscript eps enhanced color solid \"Arial\" 14\n");
		fprintf (GNUFILE,"#set output \"Mz-Opti_Results.eps\" \n");
		fprintf (GNUFILE,"#rep\n");
		fprintf (GNUFILE,"#set output\n");
		fprintf (GNUFILE,"#set term win \n");
		fclose  (GNUFILE);

      if      ( model == 1 ) myContinusFormula->ContinusFormula_Terminate ();
      else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_Terminate ();

#ifndef GUI
//
// --- write best solution to DesignResults.dat and terminate model class
//
		get_variables( design );
		MemoryStatus = 0;
		IsLoaded     = 0;
		Isallocated  = 0;
      free ( VT );
      free ( optidata );
      free ( resultdata );
      free ( PointsPerFile );
      free ( TargetDirectionPerFile );
#endif
   }
//
// --- return target
//
   return ( targetvalue );
}
void CRMOD_K_Formula_Opti::get_CornerKy ()
//----------------------------------------------------------------------
// --- get cornering stiffness
//----------------------------------------------------------------------
{
	int    i,k,j;
	double input,output1,output2,Ky1,Ky2;
	double Fy1,Fy2,Fm1,Fm2,value1,value2;
	FILE  *CORNER;
//
// --- measure ky
//
	if ( ( CORNER = fopen (DataCornerName,"w+") ) == NULL ) return;
	fprintf (CORNER,"# Load          Sim            Mea            Delta            dA\n");
	j = 0;
   for ( i=0; i<NumberOfFiles; i++ )
   {
		Fy1    =   0.0;
		Fy2    =   0.0;
		value1 =  -2.0;
		value2 =   2.0;

      for ( k=0; k<PointsPerFile[i]; k++ )
      {
         input = optidata [j+1];
			if ( TargetDirectionPerFile[i] == 2 )
         {
				output1 = optidata  [j+4]; 
				output2 = resultdata[j+1];
				if      ( input < 0.0 && input > value1 )
				{
					Fm1    = output1;
					Fy1    = output2;
					value1 = input;
				}
				else if ( input > 0.0 && input < value2 )
				{
					Fm2    = output1;
					Fy2    = output2;
					value2 = input;
				}
			}
			j = j + (Ninput+1);
      }
		if ( TargetDirectionPerFile[i] == 2 )
		{
			Ky1    = ( Fy2 - Fy1 ) / ( value2 - value1 );
			Ky2    = ( Fm2 - Fm1 ) / ( value2 - value1 );
			if ( fabs(Ky2) > 1.0e-012 ) value1 = ( Ky2 - Ky1 ) / ( 0.01 * Ky2 );
			else                        value1 = 99.0;
			fprintf (CORNER," %+12.6e %+12.6e %+12.6e %+12.6e %+12.6e \n",optidata [j+5-(Ninput+1)],Ky1,Ky2,fabs(value1),DA[i]);
		}
   }
	fclose(CORNER);
}
int CRMOD_K_Formula_Opti::ReadFile_Formula ( double *design )
//----------------------------------------------------------------------
// --- read formula data
//----------------------------------------------------------------------
{
   FILE *INPUTDATA;
   char   s[1024],*s1;
   int    i=0,l=0,linecount,varcount;
	double value;
//
// --- preset output
//
   output[0] = 0.0;
   output[1] = 0.0;
   output[2] = 0.0;
   ierr      = 0;
	varcount  = 0;
//
// --- somebody at home?
//
   if ( ( INPUTDATA = fopen (DataFileName,"r+" ) ) == NULL )
   {
      printf ( " unable to open input file DesignStart.dat for reading\n" );
      return( 0 );
   }
//
// --- parse input file
//
   fgets ( s,1024,INPUTDATA );
	if ( strstr (s,"[RMFpar1]") == NULL )
	{
#ifdef GUI
		System::Windows::Forms::MessageBox::Show(" Selected file no valied parameter data file ","Hint");
#endif
		return( 0 );
	}
//
// --- look for version and title identification
//
	if ( ( s1 = strstr (s,"version") ) != NULL  ) sscanf (s1+7,"%d",&version);   else version = 201;
	if ( ( s1 = strstr (s,"title") )   != NULL  ) strcpy (JobTitle,s1+5);        else strcpy (JobTitle,"NewFile");
//
// --- perpare for current version
//
	switch (version) 
	{
		case 201: 
		{
			Nmodelparameter = 33;
			break;
		}
		case 202: 
		{
			Nmodelparameter = 35;
			break;
		}
		case 203: 
		{
			Nmodelparameter = 42;
			break;
		}
		case 204: 
		{
			Nmodelparameter = 44;
			break;
		}
		default: 
		{
			Nmodelparameter = 33;
			break;
		}
	}
//
// --- reset design assignment (if we are in a rerun)
//
	for ( i=0; i<Nmodelparameter; i++ )
	{
		desmin[i]   = 0.0;
		desmax[i]   = 0.0;
		isdesvar[i] = 0;
	}
//
// --- read first set
//
	if ( strstr ( s,"[RMFpar1]" ) )
   {
//
// --- read parameter lines
//
      linecount = get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         input1  [varcount] = get_doublevalue ( s );
         isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
         if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
         varcount++;
      }
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar2]" ) )
   {
      linecount = get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         input1  [varcount] = get_doublevalue ( s );
         isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
         if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
         varcount++;
      }
   }
   else
   {
      printf ( " Error in data block 2 \n" );
      ierr++;
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar3]" ) )
   {
      linecount = get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         input1  [varcount] = get_doublevalue ( s );
         isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
         if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
         varcount++;
      }
   }
   else
   {
      printf ( " Error in data block 3 \n" );
      ierr++;
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar4]" ) )
   {
      linecount = get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         input1  [varcount] = get_doublevalue ( s );
         isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
         if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
         varcount++;
      }
   }
   else
   {
      printf ( " Error in data block 4 \n" );
      ierr++;
   }

   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFpar5]" ) )
   {
      linecount = get_integevalue ( s );
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,1024,INPUTDATA );
         input1  [varcount] = get_doublevalue ( s );
         isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
         if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
         varcount++;
      }
   }
   else
   {
      printf ( " Error in data block 5 \n" );
      ierr++;
   }

	if ( version >= 203 )
	{
		fgets ( s,1024,INPUTDATA );
		if ( strstr ( s,"[RMFpar6]" ) )
		{
			linecount = get_integevalue ( s );
			for ( i=0; i<linecount; i++ )
			{
				fgets ( s,1024,INPUTDATA );
				input1  [varcount] = get_doublevalue ( s );
				isdesvar[varcount] = get_designinfo  ( s,desmin+varcount,desmax+varcount );
				if ( isdesvar[varcount] ) {design[l] = input1 [varcount]; l++;}
				varcount++;
			}
		}
		else
		{
			printf ( " Error in data block 6 \n" );
			ierr++;
		}
	}
	if ( version >= 204 )
	{
// 
// --- structure for parameter LR
//
		fgets ( s,1024,INPUTDATA );
		if ( strstr ( s,"[RMFpar7]" ) )
		{
			linecount = get_integevalue ( s );
			if ( linecount != 9 ) { printf ( " Error in data block 7 \n" ); ierr++; }   
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,1,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,2,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,3,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,4,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,5,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,6,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,7,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,8,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(1,9,value);
		}
		else
		{
			printf ( " Error in data block 7 \n" );
			ierr++;
		}
// 
// --- structure for par 
//
		fgets ( s,1024,INPUTDATA );
		if ( strstr ( s,"[RMFpar8]" ) )
		{
			linecount = get_integevalue ( s );
			if ( linecount != 9 ) { printf ( " Error in data block 8\n" ); ierr++; }    
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,1,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,2,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,3,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,4,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,5,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,6,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,7,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,8,value);
			fgets ( s,1024,INPUTDATA ); s1 = strstr (s,"=")+1; sscanf (s1,"%lf",&value); myDiscreteFormula->DiscreteFormula_SetExVariables(2,9,value);
		}
		else
		{
			printf ( " Error in data block 8 \n" );
			ierr++;
		}
	}
//
// --- read model type
//
   fgets ( s,1024,INPUTDATA );
   if ( strstr ( s,"[RMFmodel]" ) )
   {
      linecount++;
      fgets ( s,1024,INPUTDATA );
      s1 = strstr ( s,"=" ) + 1;
      sscanf ( s1,"%lf",&input4 );
   }
   else
   {
      printf ( " Error in data block 6 \n" );
      ierr++;
   }

   fclose ( INPUTDATA );

   if ( varcount != Nmodelparameter )
   {
#ifdef GUI
		System::Windows::Forms::MessageBox::Show("Wrong number of input data","Error");
#else
      printf ( " number of input data = %d should be = %d\n",varcount,Nmodelparameter );
#endif
   }
   model = ( int ) input4;
   m1    = varcount;
   m2    = Ninput2;
   m3    = Ninput3;
//
// --- input check ok
//
   if ( ierr == 0 )
   {
//
// --- put version to model
//
		if      ( model == 1 ) myContinusFormula->set_version(version);
		else if ( model == 2 ) myDiscreteFormula->set_version(version);
//
// --- put data to model
//
      if      ( model == 1 ) ierr = myContinusFormula->ContinusFormula_SetVariables ( input1,m1 );
      else if ( model == 2 ) ierr = myDiscreteFormula->DiscreteFormula_SetVariables ( input1,m1 );
      else
      {
         printf ( " type of model should be 1 (continuos) or 2 (discete) and not %d\n",model );
         return ( 0 );
      }
   }
//
// --- if something went wrong
//
   if ( ierr > 0 )
   {
      printf ( " error %d while using model %d\n",ierr,model );
  }

   return ( linecount );
}
void CRMOD_K_Formula_Opti::get_variables( const double *design )
//----------------------------------------------------------------------
// --- rewrite model data file
//----------------------------------------------------------------------
{
	char *s1;
   int j=0,l=0,sliding;
   FILE *ResultFile;
//
// --- write design results in file
//
   if ( ( ResultFile = fopen ( DataResultName,"w+" ) ) == NULL )
   {
      printf ( " Can't open result file number \n" );
		return;
   }
//
// --- model variables
//
   if      ( model == 1 ) 
	{
		sliding = myContinusFormula->get_sliding ();
	}
   else if ( model == 2 ) 
	{
		sliding = myDiscreteFormula->get_sliding ();
	}

	if ( ( s1 = strstr (JobTitle,"\n") ) != NULL ) strcpy (s1,"\0");
	switch (version) 
	{
		case 201: 
		{
			fprintf ( ResultFile,"[RMFpar1] number = 4, object = stiffness version %d title %s\n",version,JobTitle);
			fprintf ( ResultFile," cx0      = %+16.10e [N/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cx1      = %+16.10e [1/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy0      = %+16.10e [N/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy1      = %+16.10e [1/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			break;
		}
		case 202: 
		{
			fprintf ( ResultFile,"[RMFpar1] number = 6, object = stiffness version %d title %s\n",version,JobTitle);
			fprintf ( ResultFile," cx0      = %+16.10e [N/mm^3] ",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cx1      = %+16.10e [1/mm^3] ",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cx2      = %+16.10e [1/Nmm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy0      = %+16.10e [N/mm^3] ",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy1      = %+16.10e [1/mm^3] ",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy2      = %+16.10e [1/Nmm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			break;
		}
		default: 
		{
			fprintf ( ResultFile,"[RMFpar1] number = 4, object = stiffness version %d title %s\n",version,JobTitle);
			fprintf ( ResultFile," cx0      = %+16.10e [N/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cx1      = %+16.10e [1/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy0      = %+16.10e [N/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			fprintf ( ResultFile," cy1      = %+16.10e [1/mm^3]",input1[j++] );
			check_design ( ResultFile,design,j,&l );
			break;
		}
	}

   fprintf ( ResultFile,"[RMFpar2] number = 3, object = offsets\n" );
   fprintf ( ResultFile," sx0      = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," sy0      = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," soff     = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );

   fprintf ( ResultFile,"[RMFpar3] number = 13, object = friction\n" );
   fprintf ( ResultFile," mue0     = %+16.10e      [-]",fabs( input1[j++] ) );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," muexy    = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," Vmuexy   = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   if ( sliding < 4 )
   {
      fprintf ( ResultFile," mue1xp   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," cmuexp   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue1xn   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," cmuexn   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue1yp   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," cmueyp   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue1yn   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," cmueyn   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
   }
   else
   {
      fprintf ( ResultFile," mue1     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue2     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue3     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue4     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue5     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," mue6     = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," frica0   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
      fprintf ( ResultFile," frica1   = %+16.10e      [-]",fabs( input1[j++] ) );
      check_design ( ResultFile,design,j,&l );
   }
   fprintf ( ResultFile," mueFz    = %+16.10e      [-]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," vslide   = %+16.10e   [mm/s]",input1[j++] );
   check_design ( ResultFile,design,j,&l );

   fprintf ( ResultFile,"[RMFpar4] number = 4, object = other\n" );
   fprintf ( ResultFile," pin      = %+16.10e [N/mm^2]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," Fzbar    = %+16.10e      [N]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," StiffSca = %+16.10e      [N]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," FrictSca = %+16.10e      [N]",input1[j++] );
   check_design ( ResultFile,design,j,&l );

   fprintf ( ResultFile,"[RMFpar5] number = 9, object = contact\n" );
   fprintf ( ResultFile," cR       = %+16.10e   [N/mm]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," R        = %+16.10e     [mm]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," Ry       = %+16.10e     [mm]",input1[j++] );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," ni       = %+16.10e      [-]", ceil(input1[j++]) );  
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," nj       = %+16.10e      [-]", ceil(input1[j++]) );  
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," ny       = %+16.10e      [-]", ceil(input1[j++]) );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," sliding  = %+16.10e      [-]", ceil(input1[j++]) );
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," pressure = %+16.10e      [-]", ceil(input1[j++]) ); 
   check_design ( ResultFile,design,j,&l );
   fprintf ( ResultFile," draw3d   = %+16.10e      [-]", ceil(input1[j++]) );
   check_design ( ResultFile,design,j,&l );

	if ( version >= 203 )
	{
		if ( version >= 204 ) fprintf ( ResultFile,"[RMFpar6] number = 11, object = contact\n" );
      else                  fprintf ( ResultFile,"[RMFpar6] number =  9, object = contact\n" );

		fprintf ( ResultFile," Dvslide    = %+16.10e   [mm/s]",input1[j++]); 
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," xpow1      = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," xpow2      = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," ypow1      = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," ypow2      = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," ypow3      = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," xygamma    = %+16.10e      [-]",input1[j++] );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," IterateFy  = %+16.10e      [-]",ceil(input1[j++]) );
		check_design ( ResultFile,design,j,&l );
		fprintf ( ResultFile," FxTurnOn   = %+16.10e      [-]",input1[j++] );

	   if ( version >= 204 )
	   {
		   check_design ( ResultFile,design,j,&l );
		   fprintf ( ResultFile," GammaCoeff = %+16.10e      [-]",input1[j++] );
		   check_design ( ResultFile,design,j,&l );
		   fprintf ( ResultFile," GammaB     = %+16.10e      [-]",input1[j++] );
		   check_design ( ResultFile,design,j,&l );
      }
      else
      {
         fprintf ( ResultFile,"\n");
      }
	}

	if ( version >= 204 )
	{
		fprintf ( ResultFile,"[RMFpar7] number = 9, object = Parameter_LR\n" );
		fprintf ( ResultFile," Parameter_LR.av     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,1));
		fprintf ( ResultFile," Parameter_LR.pv     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,2));
		fprintf ( ResultFile," Parameter_LR.kz     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,3));
		fprintf ( ResultFile," Parameter_LR.ky     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,4));
		fprintf ( ResultFile," Parameter_LR.kc     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,5));
		fprintf ( ResultFile," Parameter_LR.ac     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,6));
		fprintf ( ResultFile," Parameter_LR.az     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,7));
		fprintf ( ResultFile," Parameter_LR.p0     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,8));
		fprintf ( ResultFile," Parameter_LR.R0     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(1,9));

		fprintf ( ResultFile,"[RMFpar8] number = 9, object =  object = Par\n" );
		fprintf ( ResultFile,"  Par.R_y            = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,1));
		fprintf ( ResultFile,"  Par.p1             = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,2));
		fprintf ( ResultFile,"  Par.p2             = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,3));
		fprintf ( ResultFile,"  Par.gamma_sin      = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,4));
		fprintf ( ResultFile,"  Par.gamma_cos      = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,5));
		fprintf ( ResultFile,"  Par.gamma_comp     = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,6));
		fprintf ( ResultFile,"  Par.offset         = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,7));
		fprintf ( ResultFile,"  Par.ny             = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,8));
		fprintf ( ResultFile,"  Par.ky_stat        = %+16.10e      [-]\n",myDiscreteFormula->DiscreteFormula_GetExVariables(2,9));
	}
//
// --- model type
//
   fprintf ( ResultFile,"[RMFmodel]\n" );
   fprintf ( ResultFile," model    = %+16.10e [-]\n",( double ) model );
   fprintf ( ResultFile,"[TargetValue]\n" );
   fprintf ( ResultFile," target   = %lf \n",targetvalue );
   fprintf ( ResultFile,"[Eof]\n" );
	fprintf ( ResultFile," Dvslide max = %+16.10e   [mm/s]",myDiscreteFormula->get_vslfac()); j++; 

   fclose( ResultFile );
}
void CRMOD_K_Formula_Opti::check_design ( FILE *OUTPUT, const double *design, const int j, int *l )
//----------------------------------------------------------------------
// --- write additional design bounds if some
//----------------------------------------------------------------------
{
   int i;

   if      ( isdesvar[j-1] == 1 )
   {
      i = 0;
      if ( design[*l] > desmax[j-1] ) i= 1;
      if ( design[*l] < desmin[j-1] ) i=-1;
      *l = *l + 1;
      fprintf ( OUTPUT," designbounds  %+12.6e %+12.6e   %3d %3d\n",desmin[j-1],desmax[j-1],*l,i );
   }
   else if ( isdesvar[j-1] == 2 )
   {
      *l = *l + 1;
		if      ( input1[j-1] > 0.0 )
		{
			fprintf ( OUTPUT," designbounds  %+12.6e %+12.6e   %3d   0\n",input1[j-1]*0.5,input1[j-1]*2.0,*l );
		}
		else if ( input1[j-1] < 0.0 )
		{
			fprintf ( OUTPUT," designbounds  %+12.6e %+12.6e   %3d   0\n",input1[j-1]*2.0,input1[j-1]*0.5,*l );
		}
		else
		{
			fprintf ( OUTPUT," designbounds  %+12.6e %+12.6e   %3d   0\n",-1.0,1.0,*l );
		}
   }
   else
   {
      fprintf ( OUTPUT,"\n" );
   }
}
int CRMOD_K_Formula_Opti::Get_TargetData ()
//----------------------------------------------------------------------
// --- read measurement data
//----------------------------------------------------------------------
{
   FILE  *INPUTNAME;
   FILE  *INPUTDATA;
   char   s[1024],*s1,s2[1024];
   char   InputFileName[1024];
   int    i,ivalue1,ivalue2,ivalue3,ivalue4,ivalue5;
   double value1,value2,value3,value4,value5,value6,value7,value8;
	double GivenVelocity;
//
// --- need to refresh?
//
	if ( IsLoaded ) return (1);
//
// --- somebody at home?
//
   if ( ( INPUTNAME = fopen ( OptiFileName,"r+" ) ) == NULL )
   {
      printf ( " unable to open input file %s for reading\n",OptiFileName );
      return( 0 );
   }
//
// --- parse input file to get the size of the problem
//
   NumberOfPoints  = 0;
   fgets ( s,1024,INPUTNAME );

	if ( strstr (s,"[DataFileName and Size]") == NULL )
	{
#ifdef GUI
		System::Windows::Forms::MessageBox::Show(" Selected file no valied target data file ","Hint");
      return( 0 );
#endif
		IsLoaded = 0;
		return( 0 );
	}

   fgets ( s,1024,INPUTNAME );
//
// --- read number of data files
//
   s1 = strstr ( s,"=" ) + 1;
   sscanf ( s1,"%d",&NumberOfFiles );
	if ( ( s1 = strstr ( s1 + 1,"=" ) ) == NULL ) NumberOfAdditionalFiles = 0;
	else  sscanf ( s1+1,"%d",&NumberOfAdditionalFiles );

	if ( ! Isallocated )
	{
		if ( ( PointsPerFile = ( int * ) malloc ( sizeof( int )*(NumberOfFiles+NumberOfAdditionalFiles) ) ) == NULL )
		{
			printf ( " PointsPerFile: unable to open allocate %d int reading measurements\n",NumberOfFiles );
			return( 0 );
		}
		if ( ( TargetDirectionPerFile = ( int * ) malloc ( sizeof( int )*(NumberOfFiles+NumberOfAdditionalFiles) ) ) == NULL )
		{
			printf ( " TargetDirectionPerFile: unable to open allocate %d int reading measurements\n",NumberOfFiles );
			return( 0 );
		}

		if ( ( VT = ( double * ) malloc ( sizeof( double )*(NumberOfFiles+NumberOfAdditionalFiles) ) ) == NULL )
		{
			printf ( " unable to open allocate %d double reading measurements\n",NumberOfFiles );
			return( 0 );
		}

		if ( ( DA = ( double * ) calloc ( (NumberOfFiles+NumberOfAdditionalFiles),sizeof( double ) ) ) == NULL )
		{
			printf ( " unable to open allocate %d double reading measurements\n",NumberOfFiles );
			return( 0 );
		}
	}
//
// --- read number of points per data files
//
	i = 0;
   while ( fgets ( s,1024,INPUTNAME ) )
   {
      if ( strstr ( s,"[Eof]" ) ) break;
      if ( strstr ( s,"=" ) == NULL )
      {
         printf (" Optifile : Missing line, [Eof] or missing = at line %d in file %s\n",i,OptiFileName );
#ifdef GUI
			System::Windows::Forms::MessageBox::Show(" Optifile : Missing line, [Eof] or missing = in targetfile ","Error - GTD - 1");
#endif
        break;
      }
		else 
		{
			s1 = strstr ( s,"=" ) + 1;
		}
      if ( strstr ( s1,"=" ) == NULL )
      {
          printf (" Optifile : Missing line, [Eof] or missing = at line %d in file %s\n",i,OptiFileName );
#ifdef GUI
			System::Windows::Forms::MessageBox::Show(" Optifile : Missing line, [Eof] or missing = in targetfile ","Error - GTD - 2");
#endif
         break;
      }
		else 
		{
			s1 = strstr ( s1,"=" ) + 1;
		}
      sscanf ( s1,"%d",&ivalue1 );
      NumberOfPoints += ivalue1;
      PointsPerFile[i] = ivalue1;

      if ( strstr ( s1,"=" ) == NULL )
      {
          printf (" Optifile : Missing line, [Eof] or missing = at line %d in file %s\n",i,OptiFileName );
#ifdef GUI
			System::Windows::Forms::MessageBox::Show(" Optifile : Missing line, [Eof] or missing = in targetfile ","Error - GTD - 3");
#endif
         break;
      }
		else 
		{
			s1 = strstr ( s1,"=" ) + 1;
		}
      sscanf ( s1,"%d",&ivalue1 );
      TargetDirectionPerFile[i] = ivalue1;

      if ( strstr ( s1,"=" ) == NULL )
      {
          printf (" Optifile : Missing line, [Eof] or missing = at line %d in file %s\n",i,OptiFileName );
#ifdef GUI
			System::Windows::Forms::MessageBox::Show(" Optifile : Missing line, [Eof] or missing = in targetfile ","Error - GTD - 4");
#endif
         break;
      }
		else 
		{
			s1 = strstr ( s1,"=" ) + 1;
		}
      sscanf ( s1,"%lf",&value1 );
      VT[i] = value1;
      i++;
   }
//
// --- allocate memory
//
	if ( ! Isallocated )
	{
		if ( ( optidata = ( double * ) calloc ((Ninput+1)*(NumberOfPoints+1), sizeof(double)) ) == NULL )
		{
			printf ( " unable to open allocate %d double reading measurements\n",(Ninput+1)*NumberOfPoints );
			return( 0 );
		}

		if ( ( resultdata = ( double * ) calloc ((Noutput+1)*(NumberOfPoints+1), sizeof(double)) ) == NULL )
		{
			printf ( " unable to open allocate %d double reading measurements\n",Noutput*NumberOfPoints );
			return( 0 );
		}
	}
//
// --- parse input file and read data files
//
   ivalue2 = 0;
	ivalue4 = 0;
	vslmax  = 0.0;
   rewind ( INPUTNAME );
   fgets ( s,1024,INPUTNAME );
   fgets ( s,1024,INPUTNAME );

   while ( fgets ( s,1024,INPUTNAME ) )
   {
      if ( strstr ( s,"[Eof]" )     ) break;
		if ( strstr ( s,"=" ) == NULL ) break;
      s1 = strstr ( s,"=" ) + 1;
      sscanf  ( s1,"%s",InputFileName );
      sprintf ( s2,"%s%s",WorkDirectory,InputFileName );
      strcpy (InputFileName,s2);

      if ( ( INPUTDATA = fopen ( InputFileName,"r+" ) ) == NULL )
      {
         printf ( " unable to open input file %s for reading\n",InputFileName );
#ifdef GUI
			System::Windows::Forms::MessageBox::Show(" Data file given in targetfile not found ","Error");
#endif
         return( 0 );
      }

      s1 = strstr ( s1+1,"=" ) + 1;
      sscanf ( s1,"%d",&ivalue1 );

      s1 = strstr ( s1+1,"=" ) + 1;
      sscanf ( s1,"%d",&ivalue5 );

      s1 = strstr ( s1+1,"=" ) + 1;
      sscanf ( s1,"%lf",&GivenVelocity );
		VT[ivalue4] = fabs(GivenVelocity);

      for ( i=0; i<ivalue1; i++ )
      {
         if ( GivenVelocity < 0 ) 
			{
				ivalue3 = fscanf ( INPUTDATA,"%lf%lf%lf%lf%lf%lf%lf%lf",&value1,&value2,&value3,&value4,&value5,&value6,&value7,&value8 );
				if ( ivalue3 != Ninput )
				{
					printf ( " missing line at %d in file %s\n",i,InputFileName );
#ifdef GUI
					System::Windows::Forms::MessageBox::Show(" Data file given in targetfile out of order ","Error");
#endif
					return ( 0 );
				}
				optidata [(Ninput+1) *ivalue2  ] = value1; // long. slip
				optidata [(Ninput+1) *ivalue2+1] = value2; // late. slip
				optidata [(Ninput+1) *ivalue2+2] = value3; // turn  slip
				optidata [(Ninput+1) *ivalue2+3] = value4; // Fx
				optidata [(Ninput+1) *ivalue2+4] = value5; // Fy
				optidata [(Ninput+1) *ivalue2+5] = value6; // Fz
				optidata [(Ninput+1) *ivalue2+6] = value7; // Mz         
				optidata [(Ninput+1) *ivalue2+7] = value8; // VT
//
// --- find range of sliding velocity
//
				value2 = tan(value2*3.141529/180.0);

				if ( fabs(value8*value1) > vslmax ) vslmax = fabs(value8*value1);
				if ( fabs(value8*value2) > vslmax ) vslmax = fabs(value8*value2);
			}
			else
			{
				fgets(s,1024,INPUTDATA);
				ivalue3 = sscanf (s,"%lf%lf%lf%lf%lf%lf%lf",&value1,&value2,&value3,&value4,&value5,&value6,&value7);
				if ( ivalue3 != Ninput-1 )
				{
					printf ( " missing line at %d in file %s\n",i,InputFileName );
#ifdef GUI
					System::Windows::Forms::MessageBox::Show(" Data file given in targetfile out of order ","Error");
#endif
					return ( 0 );
				}
				optidata [(Ninput+1) *ivalue2  ] = value1;        // long. slip
				optidata [(Ninput+1) *ivalue2+1] = value2;        // late. slip
				optidata [(Ninput+1) *ivalue2+2] = value3;        // turn  slip
				optidata [(Ninput+1) *ivalue2+3] = value4;        // Fx
				optidata [(Ninput+1) *ivalue2+4] = value5;        // Fy
				optidata [(Ninput+1) *ivalue2+5] = value6;        // Fz
				optidata [(Ninput+1) *ivalue2+6] = value7;        // Mz         
				optidata [(Ninput+1) *ivalue2+7] = GivenVelocity; // VT
//
// --- find range of sliding velocity
//
				value2 = tan(value2*3.141529/180.0);

				if ( fabs(VT[ivalue4]*value1) > vslmax ) vslmax = fabs(VT[ivalue4]*value1);
				if ( fabs(VT[ivalue4]*value2) > vslmax ) vslmax = fabs(VT[ivalue4]*value2);
			}
//
// --- next data point
//
         ivalue2++;
      }
      fclose( INPUTDATA );
		ivalue4++;
   }

   value1 = vslmax / 5.0;
	myDiscreteFormula->set_vslfac(value1);
#ifdef GUI
	myDiscreteFormula->DiscreteFormula_GetVariables(input1);
	myDiscreteFormula->DiscreteFormula_Set_Friction_FB ();
#endif
	IsLoaded    = 1;
	Isallocated = 1;
   return ( 1 );
}
void CRMOD_K_Formula_Opti::freemem ( )
//----------------------------------------------------------------------
// --- free memory
//----------------------------------------------------------------------
{
	if ( Isallocated )
	{
		Isallocated = 0;
		free ( VT );
		free ( optidata );
		free ( resultdata );
		free ( PointsPerFile );
		free ( TargetDirectionPerFile );
	}
}
void CRMOD_K_Formula_Opti::get_charvalue ( const char *s, char *s2 )
//----------------------------------------------------------------------
// --- get double value out of s past next =
//----------------------------------------------------------------------
{
	int         ilen;
   const char *s1;

   s1 = strstr ( s,"=" ) - 1;
	ilen = (int)strlen(s)-(int)strlen(s1);
   strncpy (s2,s,ilen);
	s2[ilen] = '\0';
}
int CRMOD_K_Formula_Opti::get_integevalue ( const char *s )
//----------------------------------------------------------------------
// --- get integer value out of s past next =
//----------------------------------------------------------------------
{
   int  ivalue;
   const char *s1;

   s1 = strstr ( s,"=" ) + 1;
   sscanf ( s1,"%d", &ivalue );
   return ( ivalue );
}
double CRMOD_K_Formula_Opti::get_doublevalue ( const char *s )
//----------------------------------------------------------------------
// --- get double value out of s past next =
//----------------------------------------------------------------------
{
   double  value;
   const char *s1;

   s1 = strstr ( s,"=" ) + 1;
   sscanf ( s1,"%lf", &value );
   return ( value );
}
int CRMOD_K_Formula_Opti::get_designinfo ( const char *s, double *designmin, double *designmax )
//----------------------------------------------------------------------
// --- get values out of s past next 'design'
//----------------------------------------------------------------------
{
   const char *s1;

   if ( ( s1 = strstr ( s,"designbounds" ) ) == NULL  )
   {
      return ( 0 );
      *designmin = -1.0;
      *designmax =  1.0;
   }
   s1 = s1 + strlen( "designbounds" );
   sscanf ( s1,"%lf%lf",designmin,designmax );
   NumberOfDesignVariables++;
   return ( 1 );
}