#include "RMOD_K_Formula_S_function.h"

CRMOD_K_Formula_S_function RMOD_K_Formula[NTyres];

void mdlInitializeSizes(SimStruct *S)
//----------------------------------------------------------------------
// --- mdlInitializeSizes
//----------------------------------------------------------------------
{
         int     ierr,tyre;
	const double *TyreNumber;
         char    DataFileName[256];
//
// --- parameter from mask
//
	ssSetNumSFcnParams(S,2);
	if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
	{
		ssSetErrorStatus(S,"RMOD-K-Formula: parameters should by <filename tyrenumber>");
		return; 
	}
//
// --- try to get the filename
//
	mxGetString(ssGetSFcnParam(S,0),DataFileName, 256);

	if ( ! mxIsChar(ssGetSFcnParam(S,0)) ) 
	{
	  ssSetErrorStatus(S,"RMOD-K-Formula: Parameter 1 should by a string");
	  return;    
	}
//
// --- get the tyre number 
//
	TyreNumber = (const double *) mxGetData(ssGetSFcnParam(S,1));
	tyre       = (int) *TyreNumber;

	if ( tyre > NTyres-1 )
	{
	  ssSetErrorStatus(S,"RMOD-K-Formula: number of tyre to large, should be NTyres ");
	}
	else if ( tyre < 1 )
	{
	  ssSetErrorStatus(S,"RMOD-K-Formula: number of tyre to small, should be > 0 ");
	}
	else
	{
		ierr = RMOD_K_Formula[tyre-1].ReadFile_Formula (DataFileName);
		switch ( ierr )
		{
			case 0 : 
			{
				break;
			}
			case 1 : 
			{
				ssSetErrorStatus(S,"RMOD-K-Formula: data file not found");
				break;
			}
			case 2 : 
			{
				ssSetErrorStatus(S,"RMOD-K-Formula: data file with wrong number of parameter");
				break;
			}
			case 3 : 
			{
				ssSetErrorStatus(S,"RMOD-K-Formula: data file with wrong model type");
				break;
			}
		}
	}

	if (!ssSetNumInputPorts(S, NUM_INPUTS)) return;

	ssSetInputPortWidth              (S, 0, INPUT_0_WIDTH);    
	ssSetInputPortDataType           (S, 0, SS_DOUBLE);
	ssSetInputPortComplexSignal      (S, 0, INPUT_0_COMPLEX);
	ssSetInputPortDirectFeedThrough  (S, 0, INPUT_0_FEEDTHROUGH);
	ssSetInputPortRequiredContiguous (S, 0, 1);  

	ssSetInputPortWidth              (S, 1, INPUT_1_WIDTH);   
	ssSetInputPortDataType           (S, 1, SS_DOUBLE);
	ssSetInputPortComplexSignal      (S, 1, INPUT_1_COMPLEX);
	ssSetInputPortDirectFeedThrough  (S, 1, INPUT_1_FEEDTHROUGH);
	ssSetInputPortRequiredContiguous (S, 1, 1);  

	ssSetInputPortWidth              (S, 2, INPUT_2_WIDTH);    
	ssSetInputPortDataType           (S, 2, SS_DOUBLE);
	ssSetInputPortComplexSignal      (S, 2, INPUT_2_COMPLEX);
	ssSetInputPortDirectFeedThrough  (S, 2, INPUT_2_FEEDTHROUGH);
	ssSetInputPortRequiredContiguous (S, 2, 1);  

	ssSetInputPortWidth              (S, 3, INPUT_3_WIDTH);    
	ssSetInputPortDataType           (S, 3, SS_DOUBLE);
	ssSetInputPortComplexSignal      (S, 3, INPUT_3_COMPLEX);
	ssSetInputPortDirectFeedThrough  (S, 3, INPUT_3_FEEDTHROUGH);
	ssSetInputPortRequiredContiguous (S, 3, 1);  

	ssSetInputPortWidth              (S, 4, INPUT_4_WIDTH);    
	ssSetInputPortDataType           (S, 4, SS_DOUBLE);
	ssSetInputPortComplexSignal      (S, 4, INPUT_4_COMPLEX);
	ssSetInputPortDirectFeedThrough  (S, 4, INPUT_4_FEEDTHROUGH);
	ssSetInputPortRequiredContiguous (S, 4, 1);  

	if (!ssSetNumOutputPorts(S, NUM_OUTPUTS)) return;

	ssSetOutputPortWidth         (S, 0, OUTPUT_0_WIDTH);    /* Output Port 0 */
	ssSetOutputPortDataType      (S, 0, SS_DOUBLE);
	ssSetOutputPortComplexSignal (S, 0, OUTPUT_0_COMPLEX);

	ssSetOutputPortWidth         (S, 1, OUTPUT_1_WIDTH);    /* Output Port 1 */
	ssSetOutputPortDataType      (S, 1, SS_DOUBLE);
	ssSetOutputPortComplexSignal (S, 1, OUTPUT_1_COMPLEX);    

	ssSetOutputPortWidth         (S, 2, OUTPUT_2_WIDTH);    /* Output Port 2 */
	ssSetOutputPortDataType      (S, 2, SS_DOUBLE);
	ssSetOutputPortComplexSignal (S, 2, OUTPUT_2_COMPLEX);        

	ssSetNumSampleTimes(S, 1);
	ssSetNumRWork(S, 0);
	ssSetNumIWork(S, 0);
	ssSetNumPWork(S, 0);
	ssSetNumModes(S, 0);
	ssSetNumNonsampledZCs(S, 0);
	 
	ssSetInputPortDirectFeedThrough(S, 0, 1);
	ssSetInputPortDirectFeedThrough(S, 1, 1);
	ssSetInputPortDirectFeedThrough(S, 2, 1);

	ssSetOptions(S,
				   SS_OPTION_WORKS_WITH_CODE_REUSE |
				   SS_OPTION_EXCEPTION_FREE_CODE |
				   SS_OPTION_USE_TLC_WITH_ACCELERATOR);     
}
static void mdlInitializeSampleTimes(SimStruct *S)
//----------------------------------------------------------------------
// --- SampleTimes
//----------------------------------------------------------------------
{
    ssSetSampleTime(S, 0, SAMPLE_TIME_0);
    ssSetOffsetTime(S, 0, 0.0);
}
static void mdlOutputs(SimStruct *S, int_T tid)
//----------------------------------------------------------------------
// --- Outputs
//----------------------------------------------------------------------

{    
	      int     tyre;
	const double *TyreNumber;
			double  input[5], output[7];

   const double *vx = (const double*) ssGetInputPortSignal(S,0);
   const double *vy = (const double*) ssGetInputPortSignal(S,1);  
   const double *omz= (const double*) ssGetInputPortSignal(S,2);  
   const double *Fz = (const double*) ssGetInputPortSignal(S,3);  
   const double *VT = (const double*) ssGetInputPortSignal(S,4); 

         double *Fx = (      double*) ssGetOutputPortRealSignal(S,0);
         double *Fy = (      double*) ssGetOutputPortRealSignal(S,1);
         double *Mz = (      double*) ssGetOutputPortRealSignal(S,2);

	TyreNumber = (const double *) mxGetData(ssGetSFcnParam(S,1));
	tyre       = (int) *TyreNumber;

   input[0] = *vx;
   input[1] = *vy;
   input[2] = *omz;
   input[3] = *Fz;
   input[4] = *VT;

	RMOD_K_Formula[tyre-1].Evaluate_Formula (input,output);

	*Fx = output[0]; 
	*Fy = output[1]; 
	*Mz = output[2];

 //  printf (" integrator time = %lf Major = %d Minor = %d\n",ssGetT(S),ssIsMajorTimeStep(S),ssIsMinorTimeStep(S)); 
	//if ( ssIsMajorTimeStep(S) )
	//{
	//	printf ("  inputs : %lf %lf %lf %lf %lf \n",*vx,*vy,*omz,*Fz,*VT);
	//	printf (" outputs : %lf %lf %lf         \n",*Fx,*Fy,*Mz);
	//}
}
static void mdlTerminate(SimStruct *S)
//----------------------------------------------------------------------
// --- terminate 
//----------------------------------------------------------------------
{
	int           tyre;
	const double *TyreNumber;

	TyreNumber = (const double *) mxGetData(ssGetSFcnParam(S,1));
	tyre       = (int) *TyreNumber;

	RMOD_K_Formula[tyre-1].Terminate_Formula ();
}
int CRMOD_K_Formula_S_function::ReadFile_Formula ( char *Filename )
//----------------------------------------------------------------------
// --- read model data from file
//----------------------------------------------------------------------
{
   FILE   *INPUTDATA;
   char   s[256],*s1;
   int    i=0,ierr=0,linecount,varcount=0;
	double value;
//
// --- somebody at home?
//
	strcpy (DataFileName,Filename);      
	if ( ( INPUTDATA = fopen ( DataFileName,"r+" ) ) == NULL )
   {
      return (1);
   }
//
// --- parse input file
//
	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFpar1]" ) )
   {
//
// --- look for version identification
//
		s1 = strstr ( s,"Version" );
		if ( s1 != NULL ) 
		{
			sscanf (s1+7,"%d",&i);
			myContinusFormula->set_version(i);
			myDiscreteFormula->set_version(i);
		}
		else
		{
			myContinusFormula->set_version(2);
			myDiscreteFormula->set_version(2);		
		}
//
// --- read parameter lines
//
		linecount = get_integevalue (s);
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,256,INPUTDATA );
			input1[varcount++] = get_doublevalue (s);
      }
   }

	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFpar2]" ) )
   {
		linecount =+ get_integevalue (s);
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,256,INPUTDATA );
			input1[varcount++] = get_doublevalue (s);
      }
   }
	else ierr++;

	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFpar3]" ) )
   {
		linecount =+ get_integevalue (s);
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,256,INPUTDATA );
			input1[varcount++] = get_doublevalue (s);
      }
   }
	else ierr++;

	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFpar4]" ) )
   {
		linecount =+ get_integevalue (s);
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,256,INPUTDATA );
			input1[varcount++] = get_doublevalue (s);
      }
   }
	else ierr++;

	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFpar5]" ) )
   {
		linecount =+ get_integevalue (s);
      for ( i=0; i<linecount; i++ )
      {
         fgets ( s,256,INPUTDATA );
			input1[varcount++] = get_doublevalue (s);
      }
   }
	else ierr++;
//
// --- read model type
//
	fgets ( s,256,INPUTDATA );
   if ( strstr ( s,"[RMFmodel]" ) )
   {
      linecount++;
      fgets ( s,256,INPUTDATA );
      s1 = strstr ( s,"=" ) + 1;
      sscanf ( s1,"%lf",&value);
		model = (int) value;
   }
	else ierr++;

   fclose ( INPUTDATA );

//   if ( varcount != NMODELPARAMETER ) return (2);

   m1 = varcount;
//
// --- input check ok
//
	if ( ierr == 0 )
	{
//
// --- put data to model
//
		if      ( model == 1 ) ierr = myContinusFormula->ContinusFormula_SetVariables ( input1,m1 );
		else if ( model == 2 ) ierr = myDiscreteFormula->DiscreteFormula_SetVariables ( input1,m1 );
		else
		{
			return (3);
		}
	}
	else return (2);

	return (0);
}
void CRMOD_K_Formula_S_function::Evaluate_Formula (const double *input3, double *output)
//----------------------------------------------------------------------
// --- evaluate formula
//----------------------------------------------------------------------
{
   if      ( model == 1 ) myContinusFormula->ContinusFormula_Evaluate ( input3,output );
   else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_Evaluate ( input3,output );
}
void CRMOD_K_Formula_S_function::Terminate_Formula ()
//----------------------------------------------------------------------
// --- evaluate formula
//----------------------------------------------------------------------
{
   if      ( model == 1 ) myContinusFormula->ContinusFormula_Terminate ( );
   else if ( model == 2 ) myDiscreteFormula->DiscreteFormula_Terminate ( );
}
int CRMOD_K_Formula_S_function::get_integevalue (const char *s) 
//----------------------------------------------------------------------
// --- get integer value out of s past next = 
//----------------------------------------------------------------------
{
	int  ivalue;
	const char *s1;
   
	s1 = strstr ( s,"=" ) + 1;
   sscanf ( s1,"%d", &ivalue );
	return (ivalue);
}
double CRMOD_K_Formula_S_function::get_doublevalue (const char *s) 
//----------------------------------------------------------------------
// --- get double value out of s past next = 
//----------------------------------------------------------------------
{
	double  value;
	const char *s1;
   
	s1 = strstr ( s,"=" ) + 1;
   sscanf ( s1,"%lf", &value );
	return (value);
}
// Required S-function trailer
#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
